%ifarch aarch64 loongarch64 ppc64le riscv64 x86_64
%global native_compiler 1
%global natdynlink 1
%else
%global native_compiler 0
%global natdynlink 0
%endif

%global test_arches aarch64 ppc64le riscv64 x86_64

Summary:        OCaml compiler and programming environment
Name:           ocaml
Version:        5.2.0
Release:        5%{?dist}

URL:            http://www.ocaml.org
License:        QPL and (LGPLv2+ with exceptions)
Source0:        https://github.com/ocaml/ocaml/archive/%{version}.tar.gz
Source1:        macros.ocaml-rpm

# Upstream after 5.2.0:
Patch0001:      0001-Changes-synchronisation-and-consistency-with-trunk.patch
Patch0002:      0002-Changes-copy-editing.patch
# https://github.com/ocaml/ocaml/pull/13150
Patch0003:      0005-flambda-Improve-transitive-closure-in-invariant_para.patch

Patch3000:      0003-Don-t-add-rpaths-to-libraries.patch
Patch3001:      0004-configure-Allow-user-defined-C-compiler-flags.patch

Patch5000:	Add-loongarch-architecture-native-support.patch

BuildRequires:  make gcc autoconf binutils-devel perl-interpreter git
BuildRequires:  ncurses-devel gdbm-devel gawk util-linux chrpath

BuildRequires:  ocaml-srpm-macros
Requires:       ocaml-srpm-macros

Requires:       gcc
Requires:       system-rpm-config
Requires:       ocaml-runtime = %{version}-%{release}

Provides:       bundled(md5-plumb)
Provides:       ocaml(compiler) = %{version}

%global __ocaml_requires_opts -c -f '%{buildroot}%{_bindir}/ocamlrun %{buildroot}%{_bindir}/ocamlobjinfo.byte'
%global __ocaml_provides_opts -f '%{buildroot}%{_bindir}/ocamlrun %{buildroot}%{_bindir}/ocamlobjinfo.byte'


%description
OCaml is a high-level, strongly-typed, functional and object-oriented
programming language from the ML family of languages.

%package runtime
Summary:        OCaml runtime environment
Requires:       util-linux
Provides:       ocaml(runtime) = %{version}

%description runtime
This package contains the runtime environment needed to run OCaml
bytecode.


%package source
Summary:        Source code for OCaml libraries
Requires:       ocaml = %{version}-%{release}

%description source
Source code for OCaml libraries.


%package ocamldoc
Summary:        Documentation generator for OCaml
Requires:       ocaml = %{version}-%{release}
Provides:	ocamldoc

%description ocamldoc
Documentation generator for OCaml.


%package docs
Summary:        Documentation for OCaml
Requires:       ocaml = %{version}-%{release}


%description docs
This package contains man pages.


%package compiler-libs
Summary:        Compiler libraries for OCaml
Requires:       ocaml = %{version}-%{release}


%description compiler-libs
This package contains some modules used internally by the OCaml
compilers, useful for the development of some OCaml applications.
Note that this exposes internal details of the OCaml compiler which
may not be portable between versions.

%package rpm-macros
License:        LGPL-2.1-or-later WITH OCaml-LGPL-linking-exception
Summary:        RPM macros for building OCaml packages
BuildArch:      noarch
Requires:       ocaml = %{version}-%{release}
Requires:       python3


%description rpm-macros
This package provides macros for ocaml building.


%prep
%autosetup -p1 -Sgit

%ifarch loongarch64
cp /usr/lib/rpm/OpenCloudOS/config.guess ./build-aux/
cp /usr/lib/rpm/OpenCloudOS/config.sub ./build-aux/
%endif

%build
autoconf --force

sed -i '/ld_opts/s|\[\]|["%{build_ldflags}"]|' tools/ocamlmklib.ml

./configure \
    --prefix=%{_prefix} \
    --sysconfdir=%{_sysconfdir} \
    --mandir=%{_mandir} \
    --libdir=%{_libdir}/ocaml \
    --enable-flambda \
%if %{native_compiler}
    --enable-native-compiler \
    --enable-native-toplevel \
%else
    --disable-native-compiler \
    --disable-native-toplevel \
%endif
%ifarch x86_64
%if 0%{?_include_frame_pointers}
    --enable-frame-pointers \
%endif
%endif
    OC_CFLAGS='%{build_cflags}' \
    OC_LDFLAGS='%{build_ldflags}' \
    %{nil}
%make_build world
%if %{native_compiler}
%make_build opt
%make_build opt.opt
%endif

%install
%make_install
perl -pi -e "s|^%{buildroot}||" %{buildroot}%{_libdir}/ocaml/ld.conf

mkdir -p %{buildroot}%{rpmmacrodir}
install -m 0644 %{SOURCE1} %{buildroot}%{rpmmacrodir}/macros.ocaml-rpm

chrpath --delete %{buildroot}%{_libdir}/ocaml/stublibs/*.so
find %{buildroot} -name .ignore -delete
rm -f %{buildroot}%{_libdir}/ocaml/eventlog_metadata

rm -f %{buildroot}%{_defaultdocdir}/%{name}/Changes
rm -f %{buildroot}%{_defaultdocdir}/%{name}/LICENSE
rm -f %{buildroot}%{_defaultdocdir}/%{name}/README*

%check
%ifarch %{ocaml_native_compiler}
echo 'print_endline "hello, world"' > hello.ml
./ocamlopt.opt -verbose -I stdlib hello.ml -o hello ||:
%endif


%ifarch %{test_arches}
cd testsuite
make -j1 all ||:
%endif

%files
%license LICENSE
%{_bindir}/ocaml
%{_bindir}/ocamlcmt
%{_bindir}/ocamldebug
%{_bindir}/ocamlyacc
%{_bindir}/ocamlc
%{_bindir}/ocamlcp
%{_bindir}/ocamldep
%{_bindir}/ocamllex
%{_bindir}/ocamlmklib
%{_bindir}/ocamlmktop
%{_bindir}/ocamlobjinfo
%{_bindir}/ocamloptp
%{_bindir}/ocamlprof
%{_bindir}/ocamlc.byte
%{_bindir}/ocamldep.byte
%{_bindir}/ocamllex.byte
%{_bindir}/ocamlobjinfo.byte

%if %{native_compiler}
%{_bindir}/ocamlc.opt
%{_bindir}/ocamldep.opt
%{_bindir}/ocamllex.opt
%{_bindir}/ocamlobjinfo.opt
%endif

%if %{native_compiler}
%{_bindir}/ocamlnat
%{_bindir}/ocamlopt
%{_bindir}/ocamlopt.byte
%{_bindir}/ocamlopt.opt
%endif

%{_libdir}/ocaml/expunge
%{_libdir}/ocaml/ld.conf
%{_libdir}/ocaml/Makefile.config

%{_libdir}/ocaml/*.a
%if %{native_compiler}
%{_libdir}/ocaml/*.cmxa
%{_libdir}/ocaml/*.cmx
%{_libdir}/ocaml/*.o
%{_libdir}/ocaml/libasmrun_shared.so
%endif
%{_libdir}/ocaml/*.mli
%{_libdir}/ocaml/sys.ml.in
%{_libdir}/ocaml/libcamlrun_shared.so

%{_libdir}/ocaml/{dynlink,runtime_events,str,threads,unix}/*.mli
%if %{native_compiler}
%{_libdir}/ocaml/{dynlink,runtime_events,str,threads,unix}/*.a
%{_libdir}/ocaml/{dynlink,runtime_events,str,threads,unix}/*.cmxa
%{_libdir}/ocaml/{dynlink,profiling,runtime_events,str,threads,unix}/*.cmx
%{_libdir}/ocaml/profiling/*.o
%endif
%if %{natdynlink}
%{_libdir}/ocaml/{runtime_events,str,unix}/*.cmxs
%endif
%{_libdir}/ocaml/caml


%files runtime
%license LICENSE
%doc README* Changes
%{_bindir}/ocamlrun
%{_bindir}/ocamlrund
%{_bindir}/ocamlruni
%dir %{_libdir}/ocaml
%{_libdir}/ocaml/*.cmo
%{_libdir}/ocaml/*.cmi
%{_libdir}/ocaml/*.cma
%{_libdir}/ocaml/stublibs
%dir %{_libdir}/ocaml/dynlink
%{_libdir}/ocaml/dynlink/META
%{_libdir}/ocaml/dynlink/*.cmi
%{_libdir}/ocaml/dynlink/*.cma
%dir %{_libdir}/ocaml/profiling
%{_libdir}/ocaml/profiling/*.cmo
%{_libdir}/ocaml/profiling/*.cmi
%dir %{_libdir}/ocaml/runtime_events
%{_libdir}/ocaml/runtime_events/META
%{_libdir}/ocaml/runtime_events/*.cmi
%{_libdir}/ocaml/runtime_events/*.cma
%{_libdir}/ocaml/runtime-launch-info
%{_libdir}/ocaml/stdlib
%dir %{_libdir}/ocaml/str
%{_libdir}/ocaml/str/META
%{_libdir}/ocaml/str/*.cmi
%{_libdir}/ocaml/str/*.cma
%dir %{_libdir}/ocaml/threads
%{_libdir}/ocaml/threads/META
%{_libdir}/ocaml/threads/*.cmi
%{_libdir}/ocaml/threads/*.cma
%dir %{_libdir}/ocaml/unix
%{_libdir}/ocaml/unix/META
%{_libdir}/ocaml/unix/*.cmi
%{_libdir}/ocaml/unix/*.cma


%files source
%license LICENSE
%{_libdir}/ocaml/*.ml
%{_libdir}/ocaml/*.cmt*
%{_libdir}/ocaml/*/*.cmt*


%files ocamldoc
%license LICENSE
%doc ocamldoc/Changes.txt
%{_bindir}/ocamldoc*
%{_libdir}/ocaml/ocamldoc


%files docs
%{_mandir}/man1/*
%{_mandir}/man3/*


%files compiler-libs
%license LICENSE
%{_libdir}/ocaml/compiler-libs

%files rpm-macros
%{rpmmacrodir}/macros.ocaml-rpm


%changelog
* Thu Sep 26 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.2.0-5
- Rebuilt for clarifying the packages requirement in BaseOS and AppStream

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.2.0-4
- Rebuilt for loongarch release

* Thu Jun 20 2024 cunshunxia <cunshunxia@tencent.com> - 5.2.0-3
- [Type] other
- [DESC] add requires and buildrequires of ocaml-srpm-macros.

* Wed Jun 19 2024 yala <zhaojunchao@loongson.cn> - 5.2.0-2
- [Type] other
- [DESC] Add loongarch architecture native support

* Thu Jun 13 2024 cunshunxia <cunshunxia@tencent.com> - 5.2.0-1
- Upgrade to version 5.2.0

* Mon May 20 2024 Huang Yang <huangyang@loongson.cn> - 5.0.0-3
- [Type] other
- [DESC] update config.guess and config.sub on loongarch

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.0.0-2
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Aug 1 2023 Shuo Wang <abushwang@tencent.com> - 5.0.0-1
- update to 5.0.0

* Fri May 05 2023 cunshunxia <cunshunxia@tencent.com> - 4.13.1-4
- remove unused files.

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 4.13.1-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 4.13.1-2
- Rebuilt for OpenCloudOS Stream 23

* Thu Mar 09 2023 cunshunxia <cunshunxia@tencent.com> - 4.13.1-1
- initial build
